#!/usr/bin/python

# http://pythonhosted.org/pyModbusTCP/
# disembowel client.py to get rid of server etc.
from pyModbusTCP.client import ModbusClient

class BMRModbusAPI:
    "BMR HC64 regulation client API using pyModbusTCP"

    def __init__(self, channels, port=502, host='192.168.51.3'):
        # configure host in your router
        self.channels = channels
        self.version = '0.1.0'
        self.port = port
        self.host = host
        self.client = ModbusClient(host=self.host, port=self.port, timeout=0.1)

    def read_registers(self, address=0, length=1):
        if not self.client.is_open():
            if not self.client.open():
                Exception("unable to connect to " + self.host +\
                        ":" + str(self.port))
        if self.client.is_open():
            try:
                return self.client.read_holding_registers(address, length)
            except:
                return None

    def get_room_temp(self, n):
        r = self.read_registers(2032+n*6+1)
        return r and r[0] / 10.

    def get_room_status(self, n):
        r = self.read_registers(2032+n*6+2)
        return r and r[0] / 256

    def read_circuit(self, n):
        # 32 circuits, start at 0, 25 registers per circuit
        data = self.read_registers(n*25, 25)
        return {
            'name': data[:14],
            'mFlag': data[14],
            'isActive': data[15],
            'maxUserOffset': 0,
            'EndElement': 0,
            'VirtualCircuit': 0,
            'OutputType': 0,
            'WindowType': 0,
            'WindowTemperature': 0,
            'CardType': 0,
            'CardInhibition': 0,
            'CardTemperature': 0,
            'FloorMinTemp': 0,
            'Mode 1': 0,
            'Mode 2': 0,
            'Mode 3': 0,
            'Mode 4': 0,
            'Mode 5': 0,
            'BeginWeek': 0,
            'Group': 0,
            'Correction': 0
        }

    def read_group(self, n):
        # 800, 16 x 28 registers
        assert n < 16 and n >= 0

        data = self.read_registers(800 + n*28, 28)
        return {
            'name': data[:13],
            'mFlag': data[14],
            'isActive': data[15],
            'maxUserOffset': data[16],
            'endElement': data[17],
            'dummy1': '',
            'outputType': data[19],
            'windowType': 0,
            'windowTemperature': 0,
            'cardType': 0,
            'cardInhibition': 0,
            'cardTemperature': 0,
            'floorMinTemp': 0,
            'mode 1': 0,
            'mode 2': 0,
            'mode 3': 0,
            'mode 4': 0,
            'mode 5': 0,
            #...
            'BeginWeek': 0,
        }

    def write_group(self, n, data):
        assert n < 16 and n >= 0
        raise NotImplementedError

    def read_mode(self):
        # 1248, 32x23 registers
        pass

    def write_mode(self, data):
        # 1248
        # test data
        raise NotImplementedError

    def read_web_login(self):
        # 2001, 1x14 registers
        return {
            'username': '',
            'password': ''
        }

    def write_web_login(self, username='', password=''):
        # 2001, test data
        raise NotImplementedError

    def read_WTR01(self):
        # 2015, 1x2 registers
        pass

    def read_LOW(self):
        # 2017, 1x8 registers
        pass

    def read_unit(self):
        # 2025, 1x2 registers
        pass

    def read_datetime(self):
        # 2027, 1x3 registers
        pass

    def read_WTR_01_info(self):
        # 2030, 1x2 registers
        pass

    def read_room_info(self, n):
        assert n >= 0 and n < 32
        # 2032, 32x6 registers
        pass

    def write_room_info(self, n, data):
        assert n >= 0 and n < 32
        return {
            "data" : data,
            "status": "written"
        }

    def read_end_elements(self):
        # 2224, 8x2 registers
        pass

    def read_service_1(self):
        # 2240, 1x6 registers
        # MAC address
        pass

    def read_switch_module(self, n):
        # 2246, 1x4 registers
        pass

    def read_blinds_service(self):
        # 2250, 1x27 registers
        pass

    def read_blinds(self, n):
        # 2277, 32x17 registers
        assert n >= 0 and n < 32
        pass

    def read_blinds_mode(self, n):
        # 2821, 8x15 registers
        assert n >= 0 and n < 8
        pass

    def read_central(self):
        # 2941, 1x2 registers
        pass

    def read_blinds_info(self, n):
        # 2943, 32x5 registers
        assert n >= 0 and n < 32
        pass

    def read_service_2(self):
        # 3103, 1x7 registers
        pass

    def read_heat_recovery(self):
        # 3110, 1x19 registers
        pass

    def read_mode(self, n):
        # 3129, 8x23 registers
        assert n >= 0 and n < 8
        pass

    def read_transition(self):
        # 3313, 1x4 registers
        pass

    def read_global_KC(self, n):
        # 3317, 2x2 registers
        assert n >= 0 and n < 2
        pass

    def read_co2_sensor(self, n):
        # 3321, 8x5 registers
        assert n >= 0 and n < 8
        pass

    def read_ventilation_time(self, n):
        # 3800, 8x8 registers
        assert n >= 0 and n < 8
        pass

    def read_service(self):
        data = self.read_registers(1984, 17)
        return {
            'UnitAddress': data[:2],
            'HeatingSeason': 0,
            'NumberOfCircuits': 0,
            'NumberOfKC': 0,
            'TariffIndication': 0,
            'Hysteresis': 0,
            'Cooling': 0,
            'CoolingTemp': 0,
            'EthernetIP': '',
            'EthernetMask': '',
            'EthernetGate': '',
            'WebPort': '',
            'ModPort': '',
            'EthernetSwitch': '',
            'WebSwitch': '',
            'ModSwitch': '',
            'BackComp': '',
            'LowModem': '',
            'Version': '',
            'Reserved1': None,
            'Language': data[31] and 'en' or 'cz',
            'Reserved2': None,
            'Reserved4': None
        }

    def get_low(self):
        r = self.read_registers
        return {
            'status': r(2017)[0] and 'active' or 'inactive',
            'temp': r(2018)[0],
            'from_year': r(2019)[0],
            'from_month': r(2020)[0] / 256,
            'from_day': r(2020)[0] % 256,
            'from_hour': r(2021)[0] / 256,
            'from_minute': r(2021)[0] % 256,
            'till_year': r(2022)[0],
            'till_month': r(2022)[0] / 256,
            'till_day': r(2023)[0] % 256,
            'till_hour': r(2024)[0] / 256,
            'till_minute': r(2025)[0] % 256
        }

    def get_hdo_status(self):
        r = self.read_registers(2026)
        return r and (r[0] and 'low' or 'high')

    def get_mode(self):
        m = {
            0: 'summer',
            1: 'heating',
            2: 'cooling'
        }
        r = self.read_registers(1984)
        return r and m.get(r[0], 'unknown')

    def get_all_realtime_data(self):
        d = {'channels': []}
        for i in range(len(self.channels)):
            d['channels'].append({'temp': self.get_room_temp(i)})
        d['hdo'] = self.get_hdo_status()
        d['low'] = self.get_low()
        d['mode'] = self.get_mode()
        return d

    def get_hdo(self):
        raise NotImplementedError


if __name__ == '__main__':
    import sys
    channels = range(8)
    api = BMRModbusAPI(channels)
    if '-s' in sys.argv:
        import sqlite3
        dbf = sys.argv[sys.argv.index('-s')+1]
        conn = sqlite3.connect(dbf)
        cur = conn.cursor()
        for i in channels:
            rs = api.get_room_status(i)
            rt = api.get_room_temp(i)
            if rt:
                r = cur.execute('INSERT INTO bmr (channel, heating, temp) VALUES (?, ?, ?)',
                        (i, rs, rt))
        conn.commit()
